# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(scenario: str) -> str:
    return os.path.join(
        os.path.join(test_path(), "default_net_behavior"), f"docker-compose_{scenario}.yaml"
    )


class TestComposeDefaultNetBehavior(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        ('no_nets', 'default_net_behavior_default'),
        ('one_net', 'default_net_behavior_net0'),
        ('two_nets', 'podman'),
        ('with_default', 'default_net_behavior_default'),
        ('no_nets_compat', 'default_net_behavior_default'),
        ('one_net_compat', 'default_net_behavior_default'),
        ('two_nets_compat', 'default_net_behavior_default'),
        ('with_default_compat', 'default_net_behavior_default'),
    ])
    def test_nethost(self, scenario: str, default_net: str) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [podman_compose_path(), "-f", compose_yaml_path(scenario), "up", "-d"],
            )

            container_id_out, _ = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(scenario),
                    "ps",
                    "--format",
                    '{{.ID}}',
                ],
            )
            container_id = container_id_out.decode('utf-8').split('\n')[0]
            output, _ = self.run_subprocess_assert_returncode(
                [
                    "podman",
                    "inspect",
                    container_id,
                    "--format",
                    "{{range $key, $value := .NetworkSettings.Networks }}{{ $key }}\n{{ end }}",
                ],
            )
            self.assertEqual(output.decode('utf-8').strip(), default_net)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(scenario),
                "down",
                "-t",
                "0",
            ])



from argparse import Namespace
from unittest import IsolatedAsyncioTestCase
from unittest import mock

from parameterized import parameterized

from podman_compose import PullImageSettings
from podman_compose import pull_image
from podman_compose import pull_images
from podman_compose import settings_to_pull_args


class TestPullImageSettings(IsolatedAsyncioTestCase):
    def test_unsupported_policy_fallback_to_missing(self) -> None:
        settings = PullImageSettings("localhost/test:1", policy="unsupported")
        assert settings.policy == "missing"

    def test_update_policy(self) -> None:
        settings = PullImageSettings("localhost/test:1", policy="never")
        assert settings.policy == "never"

        # not supported policy
        settings.update_policy("unsupported")
        assert settings.policy == "never"

        settings.update_policy("missing")
        assert settings.policy == "missing"

        settings.update_policy("newer")
        assert settings.policy == "newer"

        settings.update_policy("always")
        assert settings.policy == "always"

        # Ensure policy is not downgraded
        settings.update_policy("build")
        assert settings.policy == "always"

    def test_pull_args(self) -> None:
        settings = PullImageSettings("localhost/test:1", policy="always", quiet=True)
        assert settings_to_pull_args(settings) == [
            "--policy",
            "always",
            "--quiet",
            "localhost/test:1",
        ]

        settings.quiet = False
        assert settings_to_pull_args(settings) == ["--policy", "always", "localhost/test:1"]

    @mock.patch("podman_compose.Podman")
    async def test_pull_success(self, podman_mock: mock.Mock) -> None:
        settings = PullImageSettings("localhost/test:1", policy="always", quiet=True)

        run_mock = mock.AsyncMock(return_value=0)
        podman_mock.run = run_mock

        result = await pull_image(podman_mock, settings)
        assert result == 0
        run_mock.assert_called_once_with(
            [], "pull", ["--policy", "always", "--quiet", "localhost/test:1"]
        )

    @mock.patch("podman_compose.Podman")
    async def test_pull_failed(self, podman_mock: mock.Mock) -> None:
        settings = PullImageSettings(
            "localhost/test:1",
            policy="always",
            quiet=True,
            ignore_pull_error=True,
        )

        podman_mock.run = mock.AsyncMock(return_value=1)

        # with ignore_pull_error=True, should return 0 even if pull fails
        result = await pull_image(podman_mock, settings)
        assert result == 0

        # with ignore_pull_error=False, should return the actual error code
        settings.ignore_pull_error = False
        result = await pull_image(podman_mock, settings)
        assert result == 1

    @mock.patch("podman_compose.Podman")
    async def test_pull_with_never_policy(self, podman_mock: mock.Mock) -> None:
        settings = PullImageSettings(
            "localhost/test:1",
            policy="never",
            quiet=True,
            ignore_pull_error=True,
        )

        run_mock = mock.AsyncMock(return_value=1)
        podman_mock.run = run_mock

        result = await pull_image(podman_mock, settings)
        assert result == 0
        assert run_mock.call_count == 0

    @parameterized.expand([
        (
            "Local image should not pull",
            [{"image": "localhost/a:latest"}],
            [],
        ),
        (
            "Remote image should pull",
            [{"image": "ghcr.io/a:latest"}],
            [
                mock.call([], "pull", ["--policy", "missing", "ghcr.io/a:latest"]),
            ],
        ),
        (
            "The same image in service should call once",
            [
                {"image": "ghcr.io/a:latest"},
                {"image": "ghcr.io/a:latest"},
                {"image": "ghcr.io/b:latest"},
            ],
            [
                mock.call([], "pull", ["--policy", "missing", "ghcr.io/a:latest"]),
                mock.call([], "pull", ["--policy", "missing", "ghcr.io/b:latest"]),
            ],
        ),
    ])
    @mock.patch("podman_compose.Podman")
    async def test_pull_image(
        self,
        desc: str,
        services: list[dict],
        calls: list,
        podman_mock: mock.Mock,
    ) -> None:
        run_mock = mock.AsyncMock(return_value=1)
        podman_mock.run = run_mock

        assert await pull_images(podman_mock, Namespace(), services) == 0
        assert run_mock.call_count == len(calls)
        if calls:
            run_mock.assert_has_calls(calls, any_order=True)

    @mock.patch("podman_compose.Podman")
    async def test_pull_image_with_build_section(
        self,
        podman_mock: mock.Mock,
    ) -> None:
        run_mock = mock.AsyncMock(return_value=1)
        podman_mock.run = run_mock

        assert (
            await pull_images(
                podman_mock,
                Namespace(),
                [
                    {"image": "ghcr.io/a:latest", "build": {"context": "."}},
                ],
            )
            == 0
        )
        assert run_mock.call_count == 1
        run_mock.assert_called_with([], "pull", ["--policy", "missing", "ghcr.io/a:latest"])



# SPDX-License-Identifier: GPL-2.0

import contextlib
import io
import unittest
from unittest import mock

from podman_compose import PodmanComposeError
from podman_compose import main


class TestMain(unittest.TestCase):
    @mock.patch("podman_compose.asyncio.run")
    def test_main_shows_clean_error_for_podman_compose_error(self, mocked_run: mock.Mock) -> None:
        def fake_asyncio_run(coro: object) -> None:
            if hasattr(coro, "close"):
                coro.close()  # prevent un-awaited coroutine warnings in this unit test
            raise PodmanComposeError("External network [missing-net] does not exist")

        mocked_run.side_effect = fake_asyncio_run
        stderr = io.StringIO()

        with contextlib.redirect_stderr(stderr), self.assertRaises(SystemExit) as ex:
            main()

        self.assertEqual(ex.exception.code, 1)
        self.assertEqual(
            stderr.getvalue().strip(),
            "Error: External network [missing-net] does not exist",
        )



# SPDX-License-Identifier: GPL-2.0

import subprocess
import unittest
from typing import Union

from parameterized import parameterized

from podman_compose import var_interpolate


class TestVarInterpolate(unittest.TestCase):
    test_cases = [
        # 0. Substitution without braces
        ("Hello $NAME!", {"NAME": "Alice"}, "Hello Alice!", True),
        # 1. Substitution with braces
        ("Hello ${NAME}", {"NAME": "Alice"}, "Hello Alice", True),
        # 2. Unset variable (empty substitution)
        ("Hello ${NAME}", {}, "Hello ", True),
        # 3. Default if unset (:-)
        ("User: ${USER:-guest}", {}, "User: guest", True),
        # 4. Default if unset or empty (:-)
        ("User: ${USER:-guest}", {"USER": ""}, "User: guest", True),
        # 5. Default if unset (-)
        ("User: ${USER-guest}", {}, "User: guest", True),
        # 6. Default if unset but not if empty (-)
        ("User: ${USER-guest}", {"USER": ""}, "User: ", True),
        # 7. Required variable (error if unset or empty)
        ("Path: ${TEST_PATH:?TEST_PATH required}", {"TEST_PATH": "/bin"}, "Path: /bin", True),
        # 8. Required variable fails when missing
        (
            "Path: ${TEST_PATH:?TEST_PATH required}",
            {},
            ValueError("required variable TEST_PATH is missing a value: TEST_PATH required"),
            True,
        ),
        # 9. Required variable fails when empty (since :?)
        (
            "Path: ${TEST_PATH:?TEST_PATH required}",
            {"TEST_PATH": ""},
            ValueError("required variable TEST_PATH is missing a value: TEST_PATH required"),
            True,
        ),
        # 10. Required variable (no colon, fails only if unset)
        ("Config: ${CFG?missing}", {"CFG": "cfg.yaml"}, "Config: cfg.yaml", True),
        # 11. Required variable fails when unset
        (
            "Config: ${CFG?missing}",
            {},
            ValueError("required variable CFG is missing a value: missing"),
            True,
        ),
        # 12. Required variable passes even if empty (no colon)
        ("Config: ${CFG?missing}", {"CFG": ""}, "Config: ", True),
        # 13. Alternative if set (:+)
        ("Alt: ${MODE:+active}", {"MODE": "1"}, "Alt: active", True),
        # 14. Alternative if set, variable empty (still uses alt)
        ("Alt: ${MODE+active}", {"MODE": ""}, "Alt: active", True),
        # 15. Alternative if not set (+)
        ("Alt: ${MODE+active}", {}, "Alt: ", True),
        # 16. Combination of multiple vars
        ("${GREETING:-Hi}, ${NAME:-stranger}!", {}, "Hi, stranger!", True),
        # 17. Mix of required and default
        ("${ENV:?Missing ENV} - ${SERVICE:-api}", {"ENV": "prod"}, "prod - api", True),
        # 18. Default values with spaces
        ("${USER:-default user}", {}, "default user", True),
        # 19. Escaped dollar sign
        (
            "Price: $$${AMOUNT}",
            {"AMOUNT": "100"},
            "Price: $100",
            False,  # In POSIX shells, $$ becomes PID, so skip shell test
        ),
        # 20. Empty variable substitution
        ("Value: ${VAR}", {"VAR": ""}, "Value: ", True),
        # 21. Nested default (inner default resolves)
        ("${OUTER:-${INNER:-default}}", {}, "default", True),
        # 22. Nested default (inner variable exists)
        ("${OUTER:-${INNER:-default}}", {"INNER": "inner_value"}, "inner_value", True),
        # 23. Nested default (outer variable exists)
        (
            "${OUTER:-${INNER:-default}}",
            {"OUTER": "outer_value", "INNER": "inner_value"},
            "outer_value",
            True,
        ),
        # 24. Nested fallback chain
        ("${A:-${B:-${C:-final}}}", {}, "final", True),
        # 25. Nested fallback uses middle value
        ("${A:-${B:-${C:-final}}}", {"B": "mid"}, "mid", True),
        # 26. Nested fallback uses top value
        ("${A:-${B:-${C:-final}}}", {"A": "top"}, "top", True),
        # 27. Nested required variable message
        ("${MAIN:-${BACKUP:?Missing BACKUP}}", {"BACKUP": "backup_value"}, "backup_value", True),
        # 28. Nested required variable fails (inner missing)
        (
            "${MAIN:-${BACKUP:?Missing BACKUP}}",
            {},
            ValueError("required variable BACKUP is missing a value: Missing BACKUP"),
            True,
        ),
        # 29. Nested default with error in inner fallback
        ("${X:-${Y:?Y required}}", {"Y": "ok"}, "ok", True),
        # 30. Nested default that triggers inner error
        (
            "${X:-${Y:?Y required}}",
            {},
            ValueError("required variable Y is missing a value: Y required"),
            True,
        ),
        # 31. Inner nested default that is never triggered because outer value is used
        ("${X:-${Y:?Y required}}", {"X": "ok"}, "ok", True),
        # 32. Nested alternative expansion
        (
            "${VAR:+${ALT:-default_alt}}",
            {"VAR": "something", "ALT": "alt_value"},
            "alt_value",
            True,
        ),
        # 33. Nested alternative fallback
        ("${VAR:+${ALT:-default_alt}}", {"VAR": "something"}, "default_alt", True),
        # 34. Nested with unset outer, inner provides default
        ("${OUTER:-prefix_${INNER:-none}}", {}, "prefix_none", True),
        # 35. Nested with inner variable set
        ("${OUTER:-prefix_${INNER:-none}}", {"INNER": "abc"}, "prefix_abc", True),
        # 36. Outer required, inner fallback
        ("${REQ:?Missing ${ALT:-something}}", {"REQ": "value"}, "value", True),
        # 37. Outer required triggers nested message
        (
            "${REQ:?Missing ${ALT:-something}}",
            {},
            ValueError("required variable REQ is missing a value: Missing something"),
            True,
        ),
        # 38. Nested alternative chain
        ("${A:+${B:+${C:-end}}}", {"A": "1", "B": "1", "C": "final"}, "final", True),
        # 39. Nested alternative where middle missing
        ("${A:+${B:+${C:-end}}}", {"A": "1"}, "", True),
        # 40. Triple nested default with middle empty
        ("${A:-${B:-${C:-${D:-default}}}}", {"B": ""}, "default", True),
        # 41. Mixed chain: required + default + alternative
        ("${ENV:-${FALLBACK:+${ALT:-alt_value}}}", {"FALLBACK": "1"}, "alt_value", True),
        # 42. Invalid variable name in braces (starts with digit)
        (
            "${1INVALID}",
            {},
            ValueError(
                "Invalid interpolation format: ${1INVALID}."
                " You may need to escape any $ with another $"
            ),
            True,
        ),
        # 43. Invalid variable name in braces (starts with non-alphanumeric)
        (
            "${-INVALID}",
            {},
            ValueError(
                "Invalid interpolation format: ${-INVALID}."
                " You may need to escape any $ with another $"
            ),
            True,
        ),
        # 44. Invalid variable name in braces (empty)
        (
            "${}",
            {},
            ValueError(
                "Invalid interpolation format: ${}. You may need to escape any $ with another $"
            ),
            False,
        ),
        # 45. Dollar sign not followed by valid variable name (should be treated as literal)
        # Disable shell comparison since POSIX shells would treat $5 as fifth argument
        ("Price is $5", {}, "Price is $5", False),
    ]

    @parameterized.expand(test_cases)
    def test_var_interpolate(
        self,
        to_interpolate: str,
        var_values: dict[str, str],
        expected: Union[str, ValueError],
        compare_shell_evaluation: bool,
    ) -> None:
        try:
            result = var_interpolate(to_interpolate, var_values)
            self.assertEqual(result, expected)
        except ValueError as e:
            self.assertTrue(
                isinstance(expected, ValueError),
                msg=f"Expected ValueError for input: {to_interpolate}",
            )
            self.assertEqual(str(e), str(expected))

        if not compare_shell_evaluation:
            return

        # Ensure that the behavior matches the behavior of POSIX shells
        process = subprocess.Popen(
            f'echo "{to_interpolate}"',
            env=var_values,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        stdout, _ = process.communicate()
        shell_result = stdout.rstrip(b'\n').decode()
        exit_code = process.returncode
        if isinstance(expected, ValueError):
            error_msg = f"Expected non-zero return code success for input: {to_interpolate}"
            self.assertNotEqual(exit_code, 0, msg=error_msg)
        else:
            self.assertEqual(shell_result, expected)



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "extends_w_file"), "docker-compose.yml")


class TestComposeExtendsWithFile(unittest.TestCase, RunSubprocessMixin):
    def test_extends_w_file(self) -> None:  # when file is Dockerfile for building the image
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                ],
            )
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
            ])
            self.assertIn("extends_w_file_web_1", str(output))
            self.assertIn("extends_w_file_important_web_1", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0


"""Test how services can be used in the additional build contexts."""

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    """ "Returns the path to the compose file used for this test module"""
    return os.path.join(test_path(), "additional_contexts_services", "docker-compose.yml")


class TestComposeBuildAdditionalContextsServices(unittest.TestCase, RunSubprocessMixin):
    def test_build_additional_contexts_services(self) -> None:
        # Podman should automatically build the images, resolving the service dependencies.
        # The build will fail if the order is incorrect or something is unresolved.
        self.run_subprocess_assert_returncode([
            podman_compose_path(),
            "-f",
            compose_yaml_path(),
            "build",
        ])



import unittest

from podman_compose import PodmanCompose


# Test the build-time resolution of dependencies created by the service:service in
# the `build.additional_contexts` section.
class TestResolveContextDependencies(unittest.TestCase):
    def setUp(self):
        self.compose = PodmanCompose()
        # project_name is used in format_name if image is not present
        self.compose.project_name = "test_project"

    def test_resolve_context_dependencies_simple(self):
        services = {
            "service_a": {"image": "image_a"},
            "service_b": {"build": {"additional_contexts": ["img=service:service_a"]}},
        }
        self.compose._resolve_context_dependencies(services)

        build_deps = services["service_b"]["build"].get("build_deps")
        self.assertEqual(build_deps, ["service_a"])

        contexts = services["service_b"]["build"]["additional_contexts"]
        # The dependency uses the specified image name from the service
        self.assertTrue(any("img=docker://image_a" in c for c in contexts))

    def test_resolve_context_dependencies_no_image_in_target(self):
        services = {
            "service_a": {},  # No image
            "service_b": {"build": {"additional_contexts": ["img=service:service_a"]}},
        }
        self.compose._resolve_context_dependencies(services)

        contexts = services["service_b"]["build"]["additional_contexts"]
        # Default name format: localhost/project_service
        expected_image = "docker://localhost/test_project_service_a"
        self.assertTrue(any(expected_image in c for c in contexts))

    def test_circular_dependencies(self):
        services = {"service_a": {"build": {"additional_contexts": ["ctx=service:service_a"]}}}
        with self.assertRaises(ValueError):
            self.compose._resolve_context_dependencies(services)

        services = {
            "service_a": {"build": {"additional_contexts": ["ctx=service:service_b"]}},
            "service_b": {"build": {"additional_contexts": ["ctx=service:service_c"]}},
            "service_c": {"build": {"additional_contexts": ["ctx=service:service_a"]}},
        }
        with self.assertRaises(ValueError):
            self.compose._resolve_context_dependencies(services)

    def test_diamond_dependency_valid(self):
        # A depends on B and C. B depends on D. C depends on D.
        # This is a perfectly cromulent arrangement
        services = {
            "service_a": {
                "build": {
                    "additional_contexts": ["ctx1=service:service_b", "ctx2=service:service_c"]
                }
            },
            "service_b": {"build": {"additional_contexts": ["ctx=service:service_d"]}},
            "service_c": {"build": {"additional_contexts": ["ctx=service:service_d"]}},
            "service_d": {"image": "image_d"},
        }
        self.compose._resolve_context_dependencies(services)

        self.assertEqual(["service_b", "service_c"], services["service_a"]["build"]["build_deps"])
        self.assertEqual(["service_d"], services["service_b"]["build"]["build_deps"])
        self.assertEqual(["service_d"], services["service_c"]["build"]["build_deps"])



# SPDX-License-Identifier: GPL-2.0
import unittest
from typing import Any
from typing import Union

from parameterized import parameterized

from podman_compose import normalize_service


class TestNormalizeService(unittest.TestCase):
    @parameterized.expand([
        ({"test": "test"}, {"test": "test"}),
        ({"build": "."}, {"build": {"context": "."}}),
        ({"build": "./dir-1"}, {"build": {"context": "./dir-1"}}),
        ({"build": {"context": "./dir-1"}}, {"build": {"context": "./dir-1"}}),
        (
            {"build": {"dockerfile": "dockerfile-1"}},
            {"build": {"dockerfile": "dockerfile-1"}},
        ),
        (
            {"build": {"context": "./dir-1", "dockerfile": "dockerfile-1"}},
            {"build": {"context": "./dir-1", "dockerfile": "dockerfile-1"}},
        ),
        (
            {"build": {"additional_contexts": ["ctx=../ctx", "ctx2=../ctx2"]}},
            {"build": {"additional_contexts": ["ctx=../ctx", "ctx2=../ctx2"]}},
        ),
        (
            {"build": {"additional_contexts": {"ctx": "../ctx", "ctx2": "../ctx2"}}},
            {"build": {"additional_contexts": ["ctx=../ctx", "ctx2=../ctx2"]}},
        ),
    ])
    def test_simple(self, input: dict[str, Any], expected: dict[str, Any]) -> None:
        self.assertEqual(normalize_service(input), expected)

    @parameterized.expand([
        ({"test": "test"}, {"test": "test"}),
        ({"build": "."}, {"build": {"context": "./sub_dir/."}}),
        ({"build": "./dir-1"}, {"build": {"context": "./sub_dir/dir-1"}}),
        ({"build": {"context": "./dir-1"}}, {"build": {"context": "./sub_dir/dir-1"}}),
        (
            {"build": {"dockerfile": "dockerfile-1"}},
            {"build": {"context": "./sub_dir", "dockerfile": "dockerfile-1"}},
        ),
        (
            {"build": {"context": "./dir-1", "dockerfile": "dockerfile-1"}},
            {"build": {"context": "./sub_dir/dir-1", "dockerfile": "dockerfile-1"}},
        ),
        (
            {"volumes": ["./nested/relative:/mnt", "../dir-in-parent:/mnt", "..:/mnt", ".:/mnt"]},
            {
                "volumes": [
                    "./sub_dir/./nested/relative:/mnt",
                    "./sub_dir/../dir-in-parent:/mnt",
                    "./sub_dir/..:/mnt",
                    "./sub_dir/.:/mnt",
                ]
            },
        ),
        (
            {
                "volumes": [
                    {
                        "type": "bind",
                        "source": "./nested/relative",
                        "target": "/mnt",
                    }
                ]
            },
            {
                "volumes": [
                    {
                        "type": "bind",
                        "source": "./sub_dir/./nested/relative",
                        "target": "/mnt",
                    }
                ]
            },
        ),
    ])
    def test_normalize_service_with_sub_dir(
        self, input: dict[str, Any], expected: dict[str, Any]
    ) -> None:
        self.assertEqual(normalize_service(input, sub_dir="./sub_dir"), expected)

    @parameterized.expand([
        ([], []),
        (["sh"], ["sh"]),
        (["sh", "-c", "date"], ["sh", "-c", "date"]),
        ("sh", ["sh"]),
        ("sleep infinity", ["sleep", "infinity"]),
        (
            "bash -c 'sleep infinity'",
            ["bash", "-c", "sleep infinity"],
        ),
    ])
    def test_command_like(self, input: Union[list[str], str], expected: list[str]) -> None:
        for key in ['command', 'entrypoint']:
            input_service = {}
            input_service[key] = input

            expected_service = {}
            expected_service[key] = expected
            self.assertEqual(normalize_service(input_service), expected_service)



# SPDX-License-Identifier: GPL-2.0

import argparse
import unittest

from podman_compose import compose_exec_args


class TestComposeExecArgs(unittest.TestCase):
    def test_minimal(self) -> None:
        cnt = get_minimal_container()
        args = get_minimal_args()

        result = compose_exec_args(cnt, "container_name", args)
        expected = ["--interactive", "--tty", "container_name"]
        self.assertEqual(result, expected)

    def test_additional_env_value_equals(self) -> None:
        cnt = get_minimal_container()
        args = get_minimal_args()
        args.env = ["key=valuepart1=valuepart2"]

        result = compose_exec_args(cnt, "container_name", args)
        expected = [
            "--interactive",
            "--tty",
            "--env",
            "key=valuepart1=valuepart2",
            "container_name",
        ]
        self.assertEqual(result, expected)


def get_minimal_container() -> dict:
    return {}


def get_minimal_args() -> argparse.Namespace:
    return argparse.Namespace(
        T=None,
        cnt_command=None,
        env=None,
        privileged=None,
        user=None,
        workdir=None,
    )



from __future__ import annotations

import copy
import unittest
from typing import Any

from parameterized import parameterized

from podman_compose import normalize_service


class TestNormalizeServicesSimple(unittest.TestCase):
    @parameterized.expand([
        (
            {"depends_on": "my_service"},
            {"depends_on": {"my_service": {"condition": "service_started"}}},
        ),
        (
            {"depends_on": ["my_service"]},
            {"depends_on": {"my_service": {"condition": "service_started"}}},
        ),
        (
            {"depends_on": ["my_service1", "my_service2"]},
            {
                "depends_on": {
                    "my_service1": {"condition": "service_started"},
                    "my_service2": {"condition": "service_started"},
                },
            },
        ),
        (
            {"depends_on": {"my_service": {"condition": "service_started"}}},
            {"depends_on": {"my_service": {"condition": "service_started"}}},
        ),
        (
            {"depends_on": {"my_service": {"condition": "service_healthy"}}},
            {"depends_on": {"my_service": {"condition": "service_healthy"}}},
        ),
    ])
    def test_normalize_service_simple(
        self, test_case: dict[str, Any], expected: dict[str, Any]
    ) -> None:
        copy.deepcopy(test_case)
        result = normalize_service(test_case)
        self.assertEqual(result, expected)



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "seccomp"), "docker-compose.yml")


class TestComposeSeccomp(unittest.TestCase, RunSubprocessMixin):
    @unittest.skip(
        "Skip till security_opt seccomp from 'docker-compose.yml' will be able to accept a "
        "relative path of 'default.json' file. Now test works as expected but only with the "
        "absolute path."
    )
    # test if seccomp uses custom seccomp profile file 'default.json' where command mkdir is not
    # allowed
    def test_seccomp(self) -> None:
        try:
            output, _, return_code = self.run_subprocess(
                [podman_compose_path(), "-f", compose_yaml_path(), "run", "--rm", "web1"],
            )
            self.assertEqual(return_code, 1)
            self.assertIn(
                b"mkdir: can't create directory '/tmp_test': Operation not permitted", output
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0


import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestFilesystem(unittest.TestCase, RunSubprocessMixin):
    def test_compose_symlink(self) -> None:
        """The context of podman-compose.yml should come from the same directory as the file even
        if it is a symlink
        """

        compose_path = os.path.join(test_path(), "filesystem/compose_symlink/docker-compose.yml")

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
                "container1",
            ])

            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "logs",
                "--no-log-prefix",
                "--no-color",
                "container1",
            ])

            self.assertEqual(out, b'data_compose_symlink\n')

        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from packaging import version
from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "nets_test3"), "docker-compose.yml")


@unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
class TestComposeNetsTest3(unittest.TestCase, RunSubprocessMixin):
    # test if services can access the networks of other services using their respective aliases
    @parameterized.expand([
        ("nets_test3_web2_1", "web3", b"test3", 0),
        ("nets_test3_web2_1", "alias11", b"test3", 0),
        ("nets_test3_web2_1", "alias12", b"test3", 0),
        ("nets_test3_web2_1", "alias21", b"test3", 0),
        ("nets_test3_web1_1", "web3", b"test3", 0),
        ("nets_test3_web1_1", "alias11", b"test3", 0),
        ("nets_test3_web1_1", "alias12", b"test3", 0),
        # connection fails as web1 service does not know net2 and its aliases
        ("nets_test3_web1_1", "alias21", b"", 1),
    ])
    def test_nets_test3(
        self,
        container_name: str,
        network_alias_name: str,
        expected_text: bytes,
        expected_returncode: int,
    ) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                    "-d",
                ],
            )
            # check connection from different services to network aliases of web3 service
            cmd = [
                "podman",
                "exec",
                "-it",
                f"{container_name}",
                "/bin/busybox",
                "wget",
                "-O",
                "-",
                "-o",
                "/dev/null",
                f"http://{network_alias_name}:8001/index.txt",
            ]
            out, _, returncode = self.run_subprocess(cmd)
            self.assertEqual(expected_returncode, returncode)
            self.assertEqual(expected_text, out.strip())
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "interpolation"), "docker-compose.yml")


class TestComposeInterpolation(unittest.TestCase, RunSubprocessMixin):
    def test_interpolation(self) -> None:
        try:
            self.run_subprocess_assert_returncode([
                "env",
                "EXAMPLE_VARIABLE_USER=test_user",
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "logs",
            ])
            self.assertIn("EXAMPLE_VARIABLE='Host user: test_user'", str(output))
            self.assertIn("EXAMPLE_BRACES='Host user: test_user'", str(output))
            self.assertIn("EXAMPLE_COLON_DASH_DEFAULT='My default'", str(output))
            self.assertIn("EXAMPLE_DASH_DEFAULT='My other default'", str(output))
            self.assertIn("EXAMPLE_DOT_ENV='This value is from the .env file'", str(output))
            self.assertIn("EXAMPLE_EMPTY=''", str(output))
            self.assertIn("EXAMPLE_LITERAL='This is a $literal'", str(output))

            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "interpolation_labels_test_1",
            ])
            inspect_output = json.loads(output)
            labels_dict = inspect_output[0].get("Config", {}).get("Labels", {})
            self.assertIn(('TEST', 'test_labels'), labels_dict.items())
            self.assertIn(('TEST.test2', 'test2(`TEST`)'), labels_dict.items())
            self.assertIn(('test.TEST', 'TEST'), labels_dict.items())

        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanCompose(unittest.TestCase, RunSubprocessMixin):
    def test_uidmaps(self) -> None:
        compose_path = os.path.join(test_path(), "uidmaps", "docker-compose.yml")
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
            ])

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "uidmaps_touch_1",
            ])

            inspect_out = json.loads(out)
            host_config_map = inspect_out[0].get("HostConfig", {}).get("IDMappings", {})
            self.assertEqual(['0:1:1', '999:0:1'], host_config_map['UidMap'])
            self.assertEqual(['0:1:1', '999:0:1'], host_config_map['GidMap'])
        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "extends"), "docker-compose.yaml")


class TestComposeExteds(unittest.TestCase, RunSubprocessMixin):
    def test_extends_service_launch_echo(self) -> None:
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
                "echo",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "logs",
                "--no-log-prefix",
                "--no-color",
                "echo",
            ])
            self.assertEqual(output, b"Zero\n")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_extends_service_launch_echo1(self) -> None:
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
                "echo1",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "logs",
                "--no-log-prefix",
                "--no-color",
                "echo1",
            ])
            self.assertEqual(output, b"One\n")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_extends_service_launch_env1(self) -> None:
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
                "env1",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "logs",
                "--no-log-prefix",
                "--no-color",
                "env1",
            ])
            lines = output.decode('utf-8').split('\n')
            # Test selected env variables to improve robustness
            lines = sorted([
                line
                for line in lines
                if line.startswith("BAR")
                or line.startswith("BAZ")
                or line.startswith("FOO")
                or line.startswith("HOME")
                or line.startswith("PATH")
                or line.startswith("container")
            ])
            self.assertEqual(
                lines,
                [
                    'BAR=local',
                    'BAZ=local',
                    'FOO=original',
                    'HOME=/root',
                    'PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin',
                    'container=podman',
                ],
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



import json
import os
import time
import unittest
from datetime import timedelta
from io import BytesIO
from pathlib import Path

from tests.integration.test_utils import EnsureHealthcheckRun
from tests.integration.test_utils import ExecutionTime
from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(test_path(), "healthcheck", "docker-compose.yml")


class TestComposeHealthcheck(unittest.TestCase, RunSubprocessMixin):
    def _is_running(self, container_name: str) -> bool:
        output, _ = self.run_subprocess_assert_returncode([
            "podman",
            "inspect",
            container_name,
        ])
        inspect = json.load(BytesIO(output))

        self.assertEqual(len(inspect), 1)
        self.assertIn("State", inspect[0])
        self.assertIn("Running", inspect[0]["State"])
        return inspect[0]["State"]["Running"]

    def _get_health_status(self, container_name: str) -> str:
        output, _ = self.run_subprocess_assert_returncode([
            "podman",
            "inspect",
            container_name,
        ])
        inspect = json.load(BytesIO(output))

        self.assertEqual(len(inspect), 1)
        self.assertIn("State", inspect[0])
        self.assertIn("Health", inspect[0]["State"])
        self.assertIn("Status", inspect[0]["State"]["Health"])

        return inspect[0]["State"]["Health"]["Status"]

    def _create_file_for_health_test(self, file_path: str) -> Path:
        path = Path(file_path)
        path.parent.mkdir(parents=True, exist_ok=True)
        path.touch(exist_ok=True)
        return path

    def _assert_status(self, expected: str, container_name: str = "healthcheck_app_1") -> str:
        status = "unknown"
        # Total test time is ~10s: grace period (~6s) + healthcheck duration (~4s)
        # Healthcheck timing: interval 1s + timeout 1s per attempt, with 2 retries
        for _ in range(20):
            status = self._get_health_status(container_name)
            if status == expected:
                return status
            time.sleep(0.5)
        return status

    # test if healthcheck is able to change health status: starting -> healthy -> unhealthy
    def test_healthcheck(self) -> None:
        try:
            with EnsureHealthcheckRun(
                runner=self, test_case=self, container_name="healthcheck_app_1"
            ):
                # the execution time of this command must be not more than 10 seconds
                # otherwise this test case makes no sense
                with ExecutionTime(max_execution_time=timedelta(seconds=10)):
                    self.run_subprocess_assert_returncode([
                        podman_compose_path(),
                        "-f",
                        compose_yaml_path(),
                        "up",
                        "-d",
                    ])

                output, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "ps",
                ])
                self.assertIn(b"healthcheck_app_1", output)
                self.assertTrue(self._is_running("healthcheck_app_1"))

                health = self._get_health_status("healthcheck_app_1")
                self.assertEqual(health, "starting")

                # condition for a container to become 'healthy'
                file_path = os.path.join(test_path(), "healthcheck", "health-trigger", "healthy")
                path = self._create_file_for_health_test(file_path)

                health_status = self._assert_status("healthy")
                self.assertEqual(health_status, "healthy")

                # condition for a container to become 'unhealthy'
                path.unlink()
                health_status = self._assert_status("unhealthy")
                self.assertEqual(health_status, "unhealthy")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0


import os
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestLogs(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        (
            "all_services_no_flag",
            [],
            [
                b'',
                b'\x1b[1;32m[test1] |\x1b[0m \x1b[37mtest1\x1b[0m',
                b'\x1b[1;33m[test2] |\x1b[0m \x1b[37mtest2\x1b[0m',
                b'\x1b[1;34m[test3] |\x1b[0m \x1b[37mtest3\x1b[0m',
            ],
        ),
        (
            "all_services_flag_no_color",
            ["--no-color"],
            [
                b'',
                b'\x1b[0m[test1] |\x1b[0m test1',
                b'\x1b[0m[test2] |\x1b[0m test2',
                b'\x1b[0m[test3] |\x1b[0m test3',
            ],
        ),
        (
            "all_services_flag_no_log_prefix",
            ["--no-log-prefix"],
            [b'', b'\x1b[37mtest1\x1b[0m', b'\x1b[37mtest2\x1b[0m', b'\x1b[37mtest3\x1b[0m'],
        ),
        (
            "all_services_flag_no_color_no_log_prefix",
            ["--no-color", "--no-log-prefix"],
            [b'', b'test1', b'test2', b'test3'],
        ),
        (
            "one_service_no_flag",
            ["test1"],
            [b'', b'\x1b[1;32m[test1] |\x1b[0m \x1b[37mtest1\x1b[0m'],
        ),
        (
            "one_service_flag_no_color",
            ["test1", "--no-color"],
            [b'', b'\x1b[0m[test1] |\x1b[0m test1'],
        ),
        (
            "one_service_flag_no_log_prefix",
            ["test1", "--no-log-prefix"],
            [b'', b'\x1b[37mtest1\x1b[0m'],
        ),
        (
            "one_service_flag_no_color_no_log_prefix",
            ["test1", "--no-color", "--no-log-prefix"],
            [b'', b'test1'],
        ),
        (
            "two_services_no_flag",
            ["test2", "test3"],
            [
                b'',
                b'\x1b[1;32m[test2] |\x1b[0m \x1b[37mtest2\x1b[0m',
                b'\x1b[1;33m[test3] |\x1b[0m \x1b[37mtest3\x1b[0m',
            ],
        ),
        (
            "two_services_flag_no_color",
            ["test2", "test3", "--no-color"],
            [b'', b'\x1b[0m[test2] |\x1b[0m test2', b'\x1b[0m[test3] |\x1b[0m test3'],
        ),
        (
            "two_services_flag_no_log_prefix",
            ["test2", "test3", "--no-log-prefix"],
            [b'', b'\x1b[37mtest2\x1b[0m', b'\x1b[37mtest3\x1b[0m'],
        ),
        (
            "two_services_flag_no_color_no_log_prefix",
            ["test2", "test3", "--no-color", "--no-log-prefix"],
            [b'', b'test2', b'test3'],
        ),
    ])
    def test_logs(
        self, test_name: str, additional_args: list[str], expected_log: list[bytes]
    ) -> None:
        compose_path = os.path.join(test_path(), "logs/docker-compose.yml")

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
            ])
            command_args = [
                podman_compose_path(),
                "-f",
                compose_path,
                "logs",
            ]
            command_args.extend(additional_args)
            out, _ = self.run_subprocess_assert_returncode(command_args)
            lines = out.split(b'\n')
            lines.sort()
            self.assertEqual(lines, expected_log)
        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

"""
test_podman_compose_networks.py

Tests the podman networking parameters
"""

# pylint: disable=redefined-outer-name
import os
import unittest
from typing import Generator

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanComposeNetwork(RunSubprocessMixin, unittest.TestCase):
    @staticmethod
    def compose_file() -> str:
        """Returns the path to the compose file used for this test module"""
        return os.path.join(test_path(), "nets_test_ip", "docker-compose.yml")

    def teardown(self) -> Generator[None, None, None]:
        """
        Ensures that the services within the "profile compose file" are removed between
        each test case.
        """
        # run the test case
        yield

        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            self.compose_file(),
            "kill",
            "-a",
        ]
        self.run_subprocess(down_cmd)

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_networks(self) -> None:
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            self.compose_file(),
            "up",
            "-d",
            "--force-recreate",
        ]

        self.run_subprocess_assert_returncode(up_cmd)

        check_cmd = [
            podman_compose_path(),
            "-f",
            self.compose_file(),
            "ps",
            "--format",
            '"{{.Names}}"',
        ]
        out, _ = self.run_subprocess_assert_returncode(check_cmd)
        self.assertIn(b"nets_test_ip_web1_1", out)
        self.assertIn(b"nets_test_ip_web2_1", out)

        expected_wget = {
            "172.19.1.10": "test1",
            "172.19.2.10": "test1",
            "172.19.2.11": "test2",
            "web3": "test3",
            "172.19.1.13": "test4",
        }

        for service in ("web1", "web2"):
            for ip, expect in expected_wget.items():
                wget_cmd = [
                    podman_compose_path(),
                    "-f",
                    self.compose_file(),
                    "exec",
                    service,
                    "wget",
                    "-q",
                    "-O-",
                    f"http://{ip}:8001/index.txt",
                ]
                out, _ = self.run_subprocess_assert_returncode(wget_cmd)
                self.assertEqual(f"{expect}\r\n", out.decode('utf-8'))

        expected_macip = {
            "web1": {
                "eth0": ["172.19.1.10", "02:01:01:00:01:01"],
                "eth1": ["172.19.2.10", "02:01:01:00:02:01"],
            },
            "web2": {"eth0": ["172.19.2.11", "02:01:01:00:02:02"]},
        }

        for service, interfaces in expected_macip.items():
            ip_cmd = [
                podman_compose_path(),
                "-f",
                self.compose_file(),
                "exec",
                service,
                "ip",
                "addr",
                "show",
            ]
            out, _ = self.run_subprocess_assert_returncode(ip_cmd)
            for interface, values in interfaces.items():
                ip, mac = values
                self.assertIn(f"ether {mac}", out.decode('utf-8'))
                self.assertIn(f"inet {ip}/", out.decode('utf-8'))

    def test_down_with_network(self) -> None:
        try:
            self.run_subprocess_assert_returncode([
                "coverage",
                "run",
                podman_compose_path(),
                "-f",
                os.path.join(test_path(), "network", "docker-compose.yml"),
                "up",
                "-d",
            ])
            output, _, _ = self.run_subprocess(["podman", "network", "ls"])
            self.assertIn("network_mystack", output.decode())
        finally:
            self.run_subprocess_assert_returncode([
                "coverage",
                "run",
                podman_compose_path(),
                "-f",
                os.path.join(test_path(), "network", "docker-compose.yml"),
                "down",
            ])
            output, _, _ = self.run_subprocess(["podman", "network", "ls"])
            self.assertNotIn("network_mystack", output.decode())



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest
from io import BytesIO

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(
        os.path.join(
            test_path(), "merge", "reset_and_override_tags", "reset_tag_attribute_w_extends"
        ),
        "docker-compose.yml",
    )


class TestComposeResetTagAttributeWithExtends(unittest.TestCase, RunSubprocessMixin):
    def test_reset_w_extends(self) -> None:  # when file is Dockerfile for building the image
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                ],
            )
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
            ])
            self.assertIn("reset_tag_attribute_w_extends_web_1", str(output))

            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "reset_tag_attribute_w_extends_web_1",
            ])
            inspect = json.load(BytesIO(output))

            self.assertEqual(len(inspect), 1)
            self.assertIn("NetworkSettings", inspect[0])
            self.assertIn("Ports", inspect[0]["NetworkSettings"])
            self.assertEqual(len(inspect[0]["NetworkSettings"]["Ports"]), 0)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(version: str = '1') -> str:
    return os.path.join(os.path.join(test_path(), "no_recreate"), f"compose{version}.yaml")


class TestComposeNoRecreate(unittest.TestCase, RunSubprocessMixin):
    def test_no_recreate(self) -> None:
        def up(args: list[str] = [], version: str = '1') -> None:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(version),
                    "up",
                    "-t",
                    "0",
                    "-d",
                    *args,
                ],
            )

        def get_container_id() -> bytes:
            return self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
                "--format",
                '{{.ID}}',
            ])[0]

        try:
            up()

            container_id = get_container_id()

            self.assertGreater(len(container_id), 0)

            # Default behavior - up with same compose file should not recreate the container
            up()
            self.assertEqual(get_container_id(), container_id)

            # Default behavior - up with modified compose file should recreate the container
            up(version='2')
            self.assertNotEqual(get_container_id(), container_id)

            container_id = get_container_id()

            # Using --no-recreate should not recreate the container
            # even if the compose file is modified
            up(["--no-recreate"], version='1')
            self.assertEqual(get_container_id(), container_id)

            # Using --force-recreate should recreate the container
            # even if the compose file is not modified
            up(["--force-recreate"], version='1')
            self.assertNotEqual(get_container_id(), container_id)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import unittest
from pathlib import Path

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version


class TestPodmanComposeInclude(unittest.TestCase, RunSubprocessMixin):
    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_podman_compose_include(self) -> None:
        """
        Test that podman-compose can execute podman-compose -f <file> up with include
        :return:
        """
        main_path = Path(__file__).parent.parent.parent.parent

        command_up = [
            "coverage",
            "run",
            str(main_path.joinpath("podman_compose.py")),
            "-f",
            str(main_path.joinpath("tests", "integration", "include", "docker-compose.yaml")),
            "up",
            "-d",
        ]

        command_check_container = [
            "podman",
            "ps",
            "-a",
            "--filter",
            "label=io.podman.compose.project=include",
            "--format",
            '"{{.Image}}"',
        ]

        command_container_id = [
            "podman",
            "ps",
            "-a",
            "--filter",
            "label=io.podman.compose.project=include",
            "--format",
            '"{{.ID}}"',
        ]

        command_down = ["podman", "rm", "--force"]

        self.run_subprocess_assert_returncode(command_up)
        out, _ = self.run_subprocess_assert_returncode(command_check_container)
        expected_output = b'"localhost/nopush/podman-compose-test:latest"\n' * 2
        self.assertEqual(out, expected_output)
        # Get container ID to remove it
        out, _ = self.run_subprocess_assert_returncode(command_container_id)
        self.assertNotEqual(out, b"")
        container_ids = out.decode().strip().split("\n")
        container_ids = [container_id.replace('"', "") for container_id in container_ids]
        command_down.extend(container_ids)
        out, _ = self.run_subprocess_assert_returncode(command_down)
        # cleanup test image(tags)
        self.assertNotEqual(out, b"")
        # check container did not exist anymore
        out, _ = self.run_subprocess_assert_returncode(command_check_container)
        self.assertEqual(out, b"")



# SPDX-License-Identifier: GPL-2.0

import os
import unittest
from pathlib import Path

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "extends_w_empty_service"), "docker-compose.yml")


class TestComposeExtendsWithEmptyService(unittest.TestCase, RunSubprocessMixin):
    def test_extends_w_empty_service(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                ],
            )
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
            ])
            self.assertIn("extends_w_empty_service_web_1", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_podman_compose_extends_w_empty_service(self) -> None:
        """
        Test that podman-compose can execute podman-compose -f <file> up with extended File which
        includes an empty service. (e.g. if the file is used as placeholder for more complex
        configurations.)
        """
        main_path = Path(__file__).parent.parent.parent.parent

        command_up = [
            "python3",
            str(main_path.joinpath("podman_compose.py")),
            "-f",
            str(
                main_path.joinpath(
                    "tests", "integration", "extends_w_empty_service", "docker-compose.yml"
                )
            ),
            "up",
            "-d",
        ]

        self.run_subprocess_assert_returncode(command_up)



import os
import subprocess


def create_base_test_image() -> None:
    subprocess.check_call(
        ['podman', 'build', '-t', 'nopush/podman-compose-test', '.'],
        cwd=os.path.join(os.path.dirname(__file__), "base_image"),
    )


create_base_test_image()
